'use strict';

var _react = require('react');

var _react2 = babelHelpers.interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _index = require('../index');

var _index2 = babelHelpers.interopRequireDefault(_index);

describe('<Node />', function () {
  var nodeData = {
    id: 'abc123',
    name: 'mockNode',
    depth: 3,
    x: 111,
    y: 222,
    parent: {
      x: 999,
      y: 888
    }
  };

  var mockProps = {
    nodeData: nodeData,
    name: nodeData.name,
    orientation: 'horizontal',
    transitionDuration: 500,
    onClick: function onClick() {},
    circleRadius: 10,
    styles: {}
  };

  jest.spyOn(_index2.default.prototype, 'applyTransform');

  // Clear method spies on prototype after each test
  afterEach(function () {
    return jest.clearAllMocks();
  });

  it('has the correct `id` attribute value', function () {
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, mockProps));

    expect(renderedComponent.find('g').prop('id')).toBe(nodeData.id);
  });

  it('applies correct base className if `nodeData._children` is defined', function () {
    var leafNodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, mockProps));
    var nodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      nodeData: babelHelpers.extends({}, nodeData, { _children: [] })
    })));

    expect(leafNodeComponent.prop('className')).toBe('leafNodeBase');
    expect(nodeComponent.prop('className')).toBe('nodeBase');
  });

  it('applies correct <circle> styles depending on `nodeData._children`', function () {
    var fixture = {
      leafNode: {
        circle: { fill: 'blue' }
      },
      node: {
        circle: { fill: 'green' }
      }
    };
    var leafNodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      styles: fixture
    })));
    var nodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      nodeData: babelHelpers.extends({}, nodeData, { _children: [] }),
      styles: fixture
    })));

    expect(leafNodeComponent.find('circle').prop('style')).toBe(fixture.leafNode.circle);
    expect(nodeComponent.find('circle').prop('style')).toBe(fixture.node.circle);
  });

  it('applies correct node name styles depending on `nodeData._children`', function () {
    var fixture = {
      node: {
        name: { stroke: '#000' }
      },
      leafNode: {
        name: { stroke: '#fff' }
      }
    };
    var leafNodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      styles: fixture
    })));
    var nodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      nodeData: babelHelpers.extends({}, nodeData, { _children: [] }),
      styles: fixture
    })));

    expect(leafNodeComponent.find('.nodeNameBase').prop('style')).toBe(fixture.leafNode.name);
    expect(nodeComponent.find('.nodeNameBase').prop('style')).toBe(fixture.node.name);
  });

  it('applies correct node attributes styles depending on `nodeData._children`', function () {
    var fixture = {
      node: {
        attributes: { stroke: '#000' }
      },
      leafNode: {
        attributes: { stroke: '#fff' }
      }
    };
    var leafNodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      styles: fixture
    })));
    var nodeComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      nodeData: babelHelpers.extends({}, nodeData, { _children: [] }),
      styles: fixture
    })));

    expect(leafNodeComponent.find('.nodeAttributesBase').prop('style')).toBe(fixture.leafNode.attributes);
    expect(nodeComponent.find('.nodeAttributesBase').prop('style')).toBe(fixture.node.attributes);
  });

  it('applies correct `transform` prop based on its `orientation`', function () {
    var horizontalTransform = 'translate(' + nodeData.parent.y + ',' + nodeData.parent.x + ')';
    var verticalTransform = 'translate(' + nodeData.parent.x + ',' + nodeData.parent.y + ')';
    var horizontalComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, mockProps));
    var verticalComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      orientation: 'vertical'
    })));
    expect(horizontalComponent.find('g').prop('transform')).toBe(horizontalTransform);
    expect(verticalComponent.find('g').prop('transform')).toBe(verticalTransform);
  });

  it('should take an `onClick` prop', function () {
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      onClick: function onClick() {}
    })));

    expect(renderedComponent.prop('onClick')).toBeDefined();
  });

  it('handles click events and passes its nodeId to onClick handler', function () {
    var onClickSpy = jest.fn();
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      onClick: onClickSpy
    })));

    renderedComponent.simulate('click');
    expect(onClickSpy).toHaveBeenCalledWith(nodeData.id);
  });

  it('maps each `props.attributes` to a <tspan> element', function () {
    var fixture = { keyA: 'valA', keyB: 'valB' };
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      attributes: fixture
    })));
    var textNode = renderedComponent.find('text').findWhere(function (n) {
      return n.prop('className') === 'nodeAttributesBase';
    });

    expect(textNode.findWhere(function (n) {
      return n.text() === 'keyA: ' + fixture.keyA;
    }).length).toBe(1);
    expect(textNode.findWhere(function (n) {
      return n.text() === 'keyB: ' + fixture.keyB;
    }).length).toBe(1);
  });

  it('applies its own x/y coords on `transform` once mounted', function () {
    var fixture = 'translate(' + nodeData.y + ',' + nodeData.x + ')';
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, mockProps));

    expect(renderedComponent.instance().applyTransform).toHaveBeenCalledWith(fixture);
  });

  it('applies updated transform if either the `x` or `y` prop changes', function () {
    // jest.spyOn(Node.prototype, 'applyTransform');
    var updatedProps = babelHelpers.extends({}, mockProps, {
      nodeData: babelHelpers.extends({}, mockProps.nodeData, {
        x: 1,
        y: 2
      })
    });
    var initialTransform = 'translate(' + mockProps.nodeData.y + ',' + mockProps.nodeData.x + ')';
    var updatedTransform = 'translate(' + updatedProps.nodeData.y + ',' + updatedProps.nodeData.x + ')';
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, mockProps));

    expect(renderedComponent.instance().applyTransform).toHaveBeenCalledWith(initialTransform);

    renderedComponent.setProps(updatedProps);

    expect(renderedComponent.instance().applyTransform).toHaveBeenCalledWith(updatedTransform);
  });

  // TODO Find a way to meaningfully test `componentWillLeave`

  // it('regresses to its parent coords when unmounting/leaving', () => {
  //   jest.spyOn(Node.prototype, 'applyTransform');
  //   const fixture = `translate(${nodeData.parent.y},${nodeData.parent.x})`;
  //   const renderedComponent = mount(
  //     <Node
  //       {...mockProps}
  //     />
  //   );
  //
  //   renderedComponent.unmount();
  //   expect(Node.prototype.applyTransform).toHaveBeenCalledWith(fixture);
  //   expect(Node.prototype.applyTransform).toHaveBeenCalledTimes(1);
  // });
});