'use strict';

var _react = require('react');

var _react2 = babelHelpers.interopRequireDefault(_react);

var _reactTransitionGroup = require('react-transition-group');

var _enzyme = require('enzyme');

var _Node = require('../../Node');

var _Node2 = babelHelpers.interopRequireDefault(_Node);

var _Link = require('../../Link');

var _Link2 = babelHelpers.interopRequireDefault(_Link);

var _index = require('../index');

var _index2 = babelHelpers.interopRequireDefault(_index);

var _mockData = require('./mockData');

describe('<Tree />', function () {
  jest.spyOn(_index2.default.prototype, 'generateTree');
  jest.spyOn(_index2.default.prototype, 'assignInternalProperties');
  jest.spyOn(_index2.default.prototype, 'handleNodeToggle');
  jest.spyOn(_index2.default.prototype, 'collapseNode');
  jest.spyOn(_index2.default.prototype, 'expandNode');
  jest.spyOn(_index2.default.prototype, 'setInitialTreeDepth');

  // Clear method spies on prototype after each test
  afterEach(function () {
    return jest.clearAllMocks();
  });

  it('builds a tree on each render', function () {
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, { data: _mockData.mockData }));
    expect(renderedComponent.instance().generateTree).toHaveBeenCalled();
  });

  it('maps every node onto a <Node />', function () {
    var nodeCount = 3; // 1 top level node + 2 child nodes in mockData
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, { data: _mockData.mockData }));

    expect(renderedComponent.find(_Node2.default).length).toBe(nodeCount);
  });

  it('maps every parent-child relation onto a <Link />', function () {
    var linkCount = 2;
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, { data: _mockData.mockData }));

    expect(renderedComponent.find(_Link2.default).length).toBe(linkCount);
  });

  it('reassigns internal props if `props.data` changes', function () {
    // `assignInternalProperties` recurses by depth: 1 level -> 1 call
    var mockDataDepth = 2;
    var mockData2Depth = 2;
    var nextProps = {
      data: _mockData.mockData2
    };
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, { data: _mockData.mockData }));

    expect(renderedComponent.instance().assignInternalProperties).toHaveBeenCalledTimes(mockDataDepth);

    renderedComponent.setProps(nextProps);

    expect(renderedComponent.instance().assignInternalProperties).toHaveBeenCalledTimes(mockDataDepth + mockData2Depth);
  });

  it('applies the `translate` prop when specified', function () {
    var fixture = { x: 123, y: 321 };
    var expected = 'translate(' + fixture.x + ',' + fixture.y + ')';
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      translate: fixture
    }));
    expect(renderedComponent.find(_reactTransitionGroup.TransitionGroup).prop('transform')).toBe(expected);
  });

  it('mutates each node\'s `y` prop according to `depthFactor` when specified', function () {
    var depthFactor = 100;
    // const expectedY = nodeData.depth * depthFactor;
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      orientation: 'vertical',
      depthFactor: depthFactor
    }));

    var _renderedComponent$in = renderedComponent.instance().generateTree(_mockData.mockData),
        nodes = _renderedComponent$in.nodes;

    nodes.forEach(function (node) {
      expect(node.y).toBe(node.depth * depthFactor);
    });
  });

  it('passes `props.orientation` to its <Node /> and <Link /> children', function () {
    var fixture = 'vertical';
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      orientation: fixture
    }));

    expect(renderedComponent.find(_Node2.default).everyWhere(function (n) {
      return n.prop('orientation') === fixture;
    })).toBe(true);
    expect(renderedComponent.find(_Link2.default).everyWhere(function (n) {
      return n.prop('orientation') === fixture;
    })).toBe(true);
  });

  it('passes `handleNodeToggle()` to its <Node /> children as onClick prop', function () {
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, { data: _mockData.mockData }));

    expect(renderedComponent.find(_Node2.default).everyWhere(function (n) {
      return n.prop('onClick') === renderedComponent.instance().handleNodeToggle;
    })).toBe(true);
  });

  it('collapses a node\'s children when it is clicked in an expanded state', function () {
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, { data: _mockData.mockData }));
    renderedComponent.find(_Node2.default).first().simulate('click'); // collapse

    expect(_index2.default.prototype.handleNodeToggle).toHaveBeenCalledTimes(1);
    expect(_index2.default.prototype.collapseNode).toHaveBeenCalled();
  });

  it('expands a node\'s children when it is clicked in a collapsed state', function () {
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, { data: _mockData.mockData }));
    renderedComponent.find(_Node2.default).first().simulate('click'); // collapse
    renderedComponent.find(_Node2.default).first().simulate('click'); // re-expand

    expect(_index2.default.prototype.handleNodeToggle).toHaveBeenCalledTimes(2);
    expect(_index2.default.prototype.collapseNode).toHaveBeenCalled();
    expect(_index2.default.prototype.expandNode).toHaveBeenCalled();
  });

  it('does not collapse a node if `props.collapsible` is false', function () {
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      collapsible: false
    }));
    renderedComponent.find(_Node2.default).first().simulate('click');

    expect(_index2.default.prototype.handleNodeToggle).toHaveBeenCalledTimes(1);
    expect(_index2.default.prototype.collapseNode).toHaveBeenCalledTimes(0);
  });

  it('sets tree depth to `props.initialDepth` if specified', function () {
    (0, _enzyme.mount)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      initialDepth: 1
    }));

    expect(_index2.default.prototype.setInitialTreeDepth).toHaveBeenCalled();
  });

  it('adds the `.rd3t-grabbable` class if `props.zoomable`', function () {
    var zoomableComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData
    }));
    var nonZoomableComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      zoomable: false
    }));

    expect(zoomableComponent.find('.rd3t-tree-container').hasClass('rd3t-grabbable')).toBe(true);
    expect(nonZoomableComponent.find('.rd3t-tree-container').hasClass('rd3t-grabbable')).toBe(false);
  });

  it('calls the onClick callback when a node is toggled', function () {
    var onClickSpy = jest.fn();
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      onClick: onClickSpy
    }));

    renderedComponent.find(_Node2.default).first().simulate('click');

    expect(onClickSpy).toHaveBeenCalledTimes(1);
  });

  it('calls the onClick callback when `props.collapsible` is false', function () {
    var onClickSpy = jest.fn();
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      collapsible: false,
      onClick: onClickSpy
    }));

    renderedComponent.find(_Node2.default).first().simulate('click');

    expect(onClickSpy).toHaveBeenCalledTimes(1);
  });

  it('clones the clicked node\'s data & passes it to the onClick callback if defined', function () {
    var onClickSpy = jest.fn();
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, {
      data: _mockData.mockData,
      onClick: onClickSpy
    }));

    renderedComponent.find(_Node2.default).first().simulate('click');

    expect(onClickSpy).toHaveBeenCalledWith(renderedComponent.find(_Node2.default).first().prop('nodeData'));
  });
});